# Imports for get_clan_list
import csv
import urllib.request
import codecs

"""
rs3clans.py Module

Made by: John Victor

List of functions:

# Added v0.1
get_clan_list(clan_name)
- Returns a List of Members and their info in a Clan (csv.reader)
For row As: ['user_name', 'rank', 'clan_exp', 'kills']

# Added v0.1
get_clan_exp(clan_name)
- Returns Total Clan Exp of a Clan (Int)

# Added v0.1
get_user_clan_exp(user_name, clan_name)
- Returns User Clan Exp (Int)

# Added v0.1
get_user_rank(user_name, clan_name)
- Returns a User's Rank in a Clan (Str)

# Added v0.1
get_user_info(user_name, clan_name)
- Returns All available Clan info (List)
As: ['user_name', 'rank', 'clan_exp', 'kills']

# Added v0.1
get_player_count(clan_name)
- Returns Player count of a Clan (Int)

# Added v0.1
get_average_clan_exp(clan_name)
- Returns the average Clan Exp per member in a Clan (Int)


RS3 API: http://runescape.wikia.com/wiki/Application_programming_interface
"""

# Clan List .csv format:
#
# ['user_name', 'rank', 'clan_exp', 'kills']
#
# Everything is read as a string and the single quotes are included.
# Use values explictly
#
# Example:
# - Getting clan exp from a user that has 323,142,653 Clan Exp:
#
# int(user_info[2])  # Output(int): 323142653
# user_info[2]  # Output(string): '323142653'


def get_clan_list(clan_name):
    """
    Gets a clan_list using csv.reader and returns it.
    It does NOT return a List type.

    If you wish to use it as is, you will have to iterate over it,
    you can append the list into another format for usability.
    """
    url = 'http://services.runescape.com/m=clan-hiscores/members_lite.ws?clanName='  # noqa
    url += clan_name

    read_url = urllib.request.urlopen(url)

    # errors="replace" is for names that contains spaces
    clan_list = csv.reader(codecs.iterdecode(read_url, 'utf-8',
                                             errors="replace"))

    return clan_list


def get_clan_exp(clan_name):
    """
    Gets the Total clan exp from clan_name and returns it.
    Returned Value is an Integer and can be used as such normally.

    It gets the Total clan exp adding up from every member in the clan,
    thus it's not very fast since those values are not cached somewhere.
    """
    clan_list = get_clan_list(clan_name)
    total_exp = 0
    for row in clan_list:
        try:
            if (row[2] != ' Total XP'):
                total_exp += int(row[2])
        except IndexError:
            print("Error: Invalid Clan name '{}'".format(clan_name))
            return 0
    return total_exp


def get_user_clan_exp(user_name, clan_name):
    """
    Gets the total Clan Exp from a user in clan_name and returns it.
    Returned Value is an Integer and can be used as such normally.
    """
    clan_list = get_clan_list(clan_name)
    for row in clan_list:
        try:
            if (row[0].lower() == user_name.lower()):
                return int(row[2])
        except IndexError:
            print("Error: Invalid Clan name '{}'".format(clan_name))
            return 0
    print("Error: User '{}' not found in clan '{}'".format(user_name,
                                                           clan_name))
    return 0


def get_user_rank(user_name, clan_name):
    """
    Gets a user's rank in clan_name and returns it.
    Returned Value is a String and can be used as such normally.
    """
    clan_list = get_clan_list(clan_name)
    for row in clan_list:
        try:
            if (row[0].lower() == user_name.lower()):
                return str(row[1])
        except IndexError:
            print("Error: Invalid Clan name '{}'".format(clan_name))
            return 0
    print("Error: User '{}' not found in clan '{}'".format(user_name,
                                                           clan_name))
    return 0


def get_user_info(user_name, clan_name):
    """
    Gets a user's clan info from clan_name and returns it as a List.

    Elements from the List can be accessed as you would normally with a List.

    List is formatted as:
    ['user_name', 'rank', 'clan_exp', 'kills']
    """
    clan_list = get_clan_list(clan_name)
    clan_info = []
    for row in clan_list:
        try:
            if (row[0].lower() == user_name.lower()):
                clan_info.append(row[0])  # user_name
                clan_info.append(row[1])  # rank
                clan_info.append(row[2])  # clan_exp
                clan_info.append(row[3])  # kills
                return clan_info
        except IndexError:
            print("Error: Invalid Clan name '{}'".format(clan_name))
            return 0
    print("Error: User '{}' not found in clan '{}'".format(user_name,
                                                           clan_name))
    return 0


def get_player_count(clan_name):
    """
    Gets the Total player count from clan_name and returns it.
    Returned Value is an Integer and can be used as such normally.

    Since Jagex's RS3 API takes a while to register when someone leaves a clan,
    this count may take a while to update.
    """
    clan_list = get_clan_list(clan_name)
    number_of_players = 0
    if 'Clanmate' in clan_list:
        for row in clan_list:
                print(row)
                number_of_players += 1
    else:
        print("Error: Invalid Clan name '{}'".format(clan_name))
        return 0
    return number_of_players


def get_average_clan_exp(clan_name):
    """
    Gets the Average clan exp from clan_name per Member and returns it.
    Returned Value is an Integer and can be used as such normally.
    """
    number_of_players = get_player_count(clan_name)
    total_clan_exp = get_clan_exp(clan_name)
    mean_clan_exp = total_clan_exp / number_of_players
    return mean_clan_exp



from setuptools import setup

setup(
    name="rs3clans",
    description='A Python 3 module wrapper for RuneScape 3 Clan\'s API',
    long_description='''# rs3clans.py
A Python 3 module wrapper for RuneScape 3 Clan's API

Current Version: 0.3.1

***

## Setup:

```bash
python3 -m pip install rs3clans
```

***

### Usage:

* Import:

```python3
import rs3clans
```

***

* Getting a full clan list (type: csv.reader):

```python
rs3clans.get_clan_list('clan_name')
```

- You can iterate through clan lists as you normally would

***

* Getting a clan's total exp (type: int):

```python
rs3clans.get_clan_exp('clan_name')
```

***

* Getting a user's total clan exp (type: int):

```python
rs3clans.get_user_clan_exp('user_name', 'clan_name')
```

***

* Getting a user's clan rank (type: string):

```python
rs3clans.get_user_rank('user_name', 'clan_name')
```

***

* Getting a user's clan Name, Rank, Clan Exp and Kills (type: list)

```python
rs3clans.get_user_info('user_name', 'clan_name')
```

- You can use specific elements of the list as you normally would or use it as whole

***

* Getting the total player count of a Clan (type: int):

```python
rs3clans.get_player_count('clan_name')
```

***

* Getting the average clan exp per player in a Clan (type: float):

```python
get_average_clan_exp('clan_name')
```
    ''',
    long_description_content_type='text/markdown',
    version='0.3.1',
    author='John Victor',
    author_email='johnvictorfs@gmail.com',
    license='MIT',
    packages=['rs3clans'],
    zip_safe=False,
    url='https://github.com/johnvictorfs/rs3clans.py',
    classifiers = (
        "Programming Language :: Python :: 3"
    )
)
